@@ -8,7 +8,7 @@ Last updated GT July 2013
 
 """
 
-import landlab
+from landlab import RasterModelGrid
 import pylab, time
 import numpy as np
 
@@ -39,7 +39,7 @@ def main():
     next_report = time.time()+report_interval   # next time to report progress
     
     # Create and initialize a raster model grid
-    mg = landlab.RasterModelGrid(numrows, numcols, dx)
+    mg = RasterModelGrid(numrows, numcols, dx)
     
     # Set up boundaries. We'll have the right and left sides open, the top and
     # bottom closed. The water depth on the left will be 5 m, and on the right 
